package org.jenkinsci.plugins.p4_client.client;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perforce.p4java.client.IClient;
import com.perforce.p4java.core.file.FileSpecBuilder;
import com.perforce.p4java.core.file.FileSpecOpStatus;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.option.client.SyncOptions;
import com.perforce.p4java.server.IOptionsServer;

public class ConnectionHelper {

	private Logger logger = LoggerFactory.getLogger(ConnectionHelper.class);
	private final IOptionsServer connection;
	private IClient iclient;

	public ConnectionHelper(ConnectionConfig config) throws Exception {
		this.connection = ConnectionFactory.getConnection(config);
	}

	public boolean login(AuthorisationConfig auth) throws Exception {
		connection.setUserName(auth.getUsername());

		// exit early if logged in
		if (isLogin())
			return true;

		switch (auth.getType()) {
		case PASSWORD:
			String pass = auth.getPassword();
			logger.info("P4:pass " + pass);
			connection.login(pass);
			break;

		case TICKET:
			String ticket = auth.getTicketValue();
			connection.setAuthTicket(ticket);
			break;

		case TICKETPATH:
			String path = auth.getTicketPath();
			// TODO set P4TICKETS environment
			break;

		default:

		}

		return isLogin();
	}

	private boolean isLogin() throws Exception {
		String status = connection.getLoginStatus();
		logger.info("P4:login " + status);
		if (status.contains("not necessary"))
			return true;
		if (status.contains("ticket expires in"))
			return true;
		return false;
	}

	/**
	 * Sync files to workspace
	 * 
	 * @return true if new files, false if up-to-date
	 * @throws Exception
	 */
	public boolean syncFiles() throws Exception {
		// build file revision spec
		List<IFileSpec> syncFiles;
		syncFiles = FileSpecBuilder.makeFileSpecList("//...");

		// Sync revision to re-edit
		SyncOptions syncOpts = new SyncOptions();
		List<IFileSpec> syncMsg = iclient.sync(syncFiles, syncOpts);
		return !trapFileSpecs(syncMsg, "file(s) up-to-date.");
	}

	public void disconnect() throws Exception {
		connection.disconnect();
	}

	/**
	 * Validates returned FileSpec from operation. Will suppress warning
	 * messages in the ignore string.
	 * 
	 * @param fileSpecs
	 * @param ignore
	 * @throws ConverterException
	 */
	public void validateFileSpecs(List<IFileSpec> fileSpecs, String... ignore)
			throws Exception {
		for (IFileSpec fileSpec : fileSpecs) {
			if (fileSpec.getOpStatus() != FileSpecOpStatus.VALID) {
				String msg = fileSpec.getStatusMessage();

				// superfluous p4java message
				boolean unknownMsg = true;
				ArrayList<String> ignoreList = new ArrayList<String>();
				ignoreList.add("file does not exist");
				ignoreList.addAll(Arrays.asList(ignore));
				for (String istring : ignoreList) {
					if (msg.contains(istring)) {
						if (logger.isTraceEnabled()) {
							logger.trace(msg);
						}
						// its a known message
						unknownMsg = false;
					}
				}

				// check and report unknown message
				if (unknownMsg) {
					logger.warn("p4java: " + msg);
				}
			}
		}
	}

	/**
	 * Look for a message in the returned FileSpec from operation.
	 * 
	 * @param fileSpecs
	 * @param ignore
	 * @return
	 * @throws ConverterException
	 */
	public boolean trapFileSpecs(List<IFileSpec> fileSpecs, String trap)
			throws Exception {
		for (IFileSpec fileSpec : fileSpecs) {
			if (fileSpec.getOpStatus() != FileSpecOpStatus.VALID) {
				String msg = fileSpec.getStatusMessage();
				if (msg.contains(trap)) {
					return true;
				}
			}
		}
		return false;
	}
}
